unit hiLedText; {   ver 2.50 }

interface

uses Windows,KOL,Messages,Matrix,Share,Win;

type
  TDotMatrix = ( mat5x7, mat7x9, mat9x12, dos5x7 );
  TPixelSize = ( pix1x1, pix2x2, pix3x3, pix4x4, pix5x5, pix6x6, pix7x7, pix8x8, pix9x9, pix10x10, pix11x11, pix12x12, pix13x13, pix14x14, pix15x15, pix16x16 );
  TPixelShape = ( spSquare, spShaped, spRound );
  TMyBorder = ( frRaised, frLowered, frSingle, frNone );
  TPixelState = ( pixOn, pixOff );
  TPixelPtr = ^TPixelState;
  T_FileName = type string;

type
 THILedText = class(THIWin)
  private
    BitMap :PBitmap;      
    fPixelSize: TPixelSize;     { Size of a LCD pixel (in screen pixels }
    fPixelShape: TPixelShape;   { Shape of a LCD pixel }
    fDotMatrix: TDotMatrix;     { Type of character matrix on the LCD }
    fPixelSpacing: integer;     { Space between each pixel in the matrix }
    fCharSpacing: integer;      { Space between the characters on the LCD }
    fLineSpacing: integer;      { Space between text lines on the display }
    fMatrixSpacing: integer;    { Empty space of matrix for bottom }
    fBorderSpace: integer;      { Distance from the LCD border }
    fTextLines: integer;        { Number of text lines on the LCD }
    fNoOfChars: integer;        { Number of characters on a single line }
    fBackGround: TColor;        { LCD background color }
    fPixOnColor: TColor;        { LCD pixel ON color }
    fPixOffColor: TColor;       { LCD pixel OFF color }
    fPixHalfColor: TColor;      { Half intensity ON color }
    fPixWidth: integer;         { Pixel widht }
    fPixHeight: integer;        { Pixel height }
    fWidth: integer;            { Label width in pixels }
    fHeight: integer;           { Label height in pixels }
    charw, charh: integer;      { Temp. storage of character sizes }
    psx, psy: integer;          { Internal pixel width variables }
    pix_x, pix_y: integer;      { Font size }
    first_c, last_c: integer;   { First and last character to draw }
    charbuf: array[ 0..255 ] of Char;

    fFonColor,fFonHover,fPixOn,fPixOff,FPixOnHover,FPixOffHover: TColor;
    FText: string;
    FColorHover: boolean;

    procedure CalcHalfColor;
    procedure GetAsciiInterval;
    procedure CalcCharSize;

    function  GetCaption: string;
    procedure SetCaption( const Value: string );
    function  GetTextBuf( Buffer: PChar; BufSize: Integer ): Integer;
    procedure SetTextBuf( Buffer: PChar );

    procedure DrawMatrix( BitMap: PBitMap; xpos, ypos: integer; charindex: integer );
    procedure DrawCharacters( BitMap: PBitMap );
    procedure Paint;
    procedure Resize;

    procedure _OnClick( Sender: PObj );
    procedure _OnMouseEnter( Sender: PObj );
    procedure _OnMouseLeave( Sender: PObj );

  public
    _prop_Text: string; 
    _event_OnClick : THI_Event;
    _data_Text : THI_Event;

    property _prop_ColorHover: Boolean write FColorHover;
    property _prop_FonColor: TColor    write FFonColor;
    property _prop_FonHover: TColor    write FFonHover;
    property _prop_PixOnColor: TColor  write FPixOn;
    property _prop_PixOffColor: TColor write FPixOff;
    property _prop_PixOnHover: TColor  write FPixOnHover;
    property _prop_PixOffHover: TColor write FPixOffHover;

    property _prop_DotMatrix: TDotMatrix   write FDotMatrix;
    property _prop_PixelSize: TPixelSize   write FPixelSize;
    property _prop_PixelSpacing: integer   write FPixelSpacing;
    property _prop_PixelShape: TPixelShape write FPixelShape;
    property _prop_CharSpacing: integer    write FCharSpacing;
    property _prop_LineSpacing: integer    write FLineSpacing;
    property _prop_MatrixSpacing: integer  write FMatrixSpacing;
    property _prop_BorderSpace: integer    write FBorderSpace;

    procedure Init; override;
    destructor Destroy; override;
    procedure _work_doText(var _Data:TData; Index : word);
    procedure _work_doColorHover(var _Data:TData; Index : word);
    procedure _work_doFonColor(var _Data:TData; Index : word);
    procedure _work_doFonHover(var _Data:TData; Index : word);
    procedure _work_doPixOnColor(var _Data:TData; Index : word);
    procedure _work_doPixOffColor(var _Data:TData; Index : word);
    procedure _work_doPixOnHover(var _Data:TData; Index : word);
    procedure _work_doPixOffHover(var _Data:TData; Index : word);

    procedure _work_doDotMatrix(var _Data:TData; Index : word);
    procedure _work_doPixelSize(var _Data:TData; Index : word);
    procedure _work_doPixelSpacing(var _Data:TData; Index : word);
    procedure _work_doPixelShape(var _Data:TData; Index : word);
    procedure _work_doCharSpacing(var _Data:TData; Index : word);
    procedure _work_doLineSpacing(var _Data:TData; Index : word);
    procedure _work_doMatrixSpacing(var _Data:TData; Index : word);
    procedure _work_doBorderSpace(var _Data:TData; Index : word);

    procedure _var_Caption(var Data:TData; Index:word);


  end;

implementation

function StrCopy( Dest: PChar; const Source: PChar ): PChar;
asm
        PUSH    EDI
        PUSH    ESI
        MOV     ESI,EAX
        MOV     EDI,EDX
        MOV     ECX,0FFFFFFFFH
        XOR     AL,AL
        REPNE   SCASB
        NOT     ECX
        MOV     EDI,ESI
        MOV     ESI,EDX
        MOV     EDX,ECX
        MOV     EAX,EDI
        SHR     ECX,2
        REP     MOVSD
        MOV     ECX,EDX
        AND     ECX,3
        REP     MOVSB
        POP     ESI
        POP     EDI
end;

function WndProcLCDBVLCDLabel( Sender: PControl; var Msg: TMsg; var Rslt: Integer ): Boolean;
var   P: TPaintStruct;
      fControl: THILedText;
begin
   Result := False;
   fControl := THILedText(Sender.Tag); 
   if msg.message = WM_PAINT then begin
      BeginPaint( Sender.handle, P );
         fControl.BitMap.Draw( Sender.Canvas.Handle, 0, 0 );
      EndPaint( Sender.handle, P );
      Result := True;
      Rslt := 0;
   end else if msg.message = WM_SIZE then begin
      fControl.Resize;
      Result := True;
   end;
end;

procedure THILedText.init;
begin
   FText := _prop_Text;
   Control := NewPaintBox(FParent);
   Control.Tag := longint(Self);
   Control.Attachproc(WndProcLCDBVLCDLabel);
   fBackGround := FFonColor;
   fPixOnColor := FPixOn;
   fPixOffColor := FPixOff;
   CalcHalfColor; // Halftone On color
   Control.OnClick := _OnClick;
   Control.OnMouseEnter := _OnMouseEnter;
   Control.OnMouseLeave := _OnMouseLeave;
inherited;
   SetCaption(FText);
end;

destructor THILedText.Destroy;
begin
   BitMap.free;
   inherited;
end;

procedure THILedText._work_doText;
begin
   FText := ReadString( _Data,_data_Text,_prop_Text );
   SetCaption(FText);
   Paint;
end;

procedure THILedText._OnClick;
begin
   _hi_OnEvent(_event_OnClick);
end;

procedure THILedText._OnMouseEnter;
begin
   if not FColorHover then exit; 
   fBackGround := FFonHover;
   fPixOnColor := FPixOnHover;
   fPixOffColor := FPixOffHover;
   Paint;
end;

procedure THILedText._OnMouseLeave;
begin
   if not FColorHover then exit; 
   fBackGround := FFonColor;
   fPixOnColor := FPixOn;
   fPixOffColor := FPixOff;
   Paint;
end;

procedure THILedText._work_doColorHover;
begin
   FColorHover := ReadBool(_data);
end;

procedure THILedText._work_doFonColor;
begin
   FFonColor := ToInteger(_Data);
   FBackGround := FFonColor;
   Paint;
end;

procedure THILedText._work_doPixOnColor;
begin
   fPixOn := ToInteger(_Data);
   fPixOnColor := fPixOn;
   Paint;
end;

procedure THILedText._work_doPixOffColor;
begin
   fPixOff := ToInteger(_Data);
   fPixOffColor := fPixOff;
   Paint;
end;

procedure THILedText._work_doFonHover;
begin
   FFonHover := ToInteger(_Data);
end;

procedure THILedText._work_doPixOnHover;
begin
   FPixOnHover := ToInteger(_Data);
end;

procedure THILedText._work_doPixOffHover;
begin
   FPixOffHover := ToInteger(_Data);
end;

procedure THILedText._work_doDotMatrix;
begin
   FDotMatrix := TDotMatrix(ToInteger(_Data));
   Resize;
end;

procedure THILedText._work_doPixelSize;
begin
   FPixelSize := TPixelSize(ToInteger(_Data));
   Resize;
end;

procedure THILedText._work_doPixelSpacing;
begin
   FPixelSpacing := ToInteger(_Data);
   if FPixelSpacing < 0 then FPixelSpacing := 0;
   Resize;
end;

procedure THILedText._work_doPixelShape;
begin
   FPixelShape := TPixelShape(ToInteger(_Data));
   Resize;
end;

procedure THILedText._work_doCharSpacing;
begin
   FCharSpacing := ToInteger(_Data);
   if FCharSpacing < 0 then FCharSpacing := 0;
   Resize;
end;

procedure THILedText._work_doLineSpacing;
begin
   FLineSpacing := ToInteger(_Data);
   if FLineSpacing < 0 then FLineSpacing := 0;
   Resize;
end;

procedure THILedText._work_doMatrixSpacing;
begin
   FMatrixSpacing := ToInteger(_Data);
   if FMatrixSpacing < 0 then FMatrixSpacing := 0;
   Resize;
end;

procedure THILedText._work_doBorderSpace;
begin
   FBorderSpace := ToInteger(_Data);
   if FBorderSpace < 0 then FBorderSpace := 0;
   Resize;
end;
  
procedure THILedText._var_Caption;
begin
   dtString(Data, FText);
end;

procedure THILedText.Resize;
begin
   GetAsciiInterval;
   CalcCharSize;
   Paint;
end;

procedure THILedText.Paint;
var   T_Rect: TRect;
begin
   with Control{$ifndef F_P}^{$endif} do begin
{ Draw image off-screen }
        Bitmap.Canvas.Brush.Color := fBackGround;
        Bitmap.Canvas.Rectangle( 0, 0, Width, Height );
{ Border drawing }
        BitMap.Canvas.Brush.BrushStyle := bsSolid;
        if Visible then begin { Draw only real graphics if display is marked visible }
          T_Rect.Left := 0;
          T_Rect.Top := 0;
          T_Rect.Right := Width;
          T_Rect.Bottom := Height; // := BitMap.Canvas.Rect(0, 0, Width, Height);
          Bitmap.Canvas.Brush.Color := FBackGround;
          Canvas.FillRect( T_Rect ); { Background fill }
{ Character drawing }
          DrawCharacters( BitMap );
        end else begin { Invisible - draw a square in background color }
          Brush.Color := FParent.Color;
          Canvas.FillRect( Canvas.ClipRect );
        end;
{ Copy drawn characters to Window bitmap }
        Perform(WM_PAINT,-1,0);
    end;
end;

procedure THILedText.GetAsciiInterval;
begin
    case FDotMatrix of
      mat5x7: begin
          first_c := MAT5X7_FIRST;
          last_c := MAT5X7_LAST;
        end;
      mat7x9: begin
          first_c := MAT7X9_FIRST;
          last_c := MAT7X9_LAST;
        end;
      mat9x12: begin
          first_c := MAT9X12_FIRST;
          last_c := MAT9X12_LAST;
        end;
      dos5x7: begin
          first_c := DOS5X7_FIRST;
          last_c := DOS5X7_LAST;
        end;
    end;
end;

procedure THILedText.CalcHalfColor;
var
  red, green, blue, cntrl: byte;
begin
    blue := byte( FPixOnColor ) div 2;
    green := byte( FPixOnColor shr 8 ) div 2;
    red := byte( FPixOnColor shr 16 ) div 2;
    cntrl := byte( FPixOnColor shr 24 );
    FPixHalfColor := blue + ( green * $100 ) + ( red * $10000 ) + ( cntrl * $1000000 );
end;

procedure THILedText.CalcCharSize;
begin
   with Control{$ifndef F_P}^{$endif} do begin
    psx := Ord( FPixelSize ) + 1;
    psy := psx;
    FPixWidth := psx;
    FPixHeight := psy;
    case FDotMatrix of { Calculate the space taken by the character matrix }
      mat5x7: begin
          pix_x := MAT5X7_WIDTH;
          pix_y := MAT5X7_HEIGHT;
        end;
      mat7x9: begin
          pix_x := MAT7X9_WIDTH;
          pix_y := MAT7X9_HEIGHT;
        end;
      mat9x12: begin
          pix_x := MAT9X12_WIDTH;
          pix_y := MAT9X12_HEIGHT;
        end;
      dos5x7: begin
          pix_x := DOS5X7_WIDTH;
          pix_y := DOS5X7_HEIGHT;
        end;
    end;
    pix_y := pix_y + FMatrixSpacing;
    charw := ( pix_x * psx ) + ( ( pix_x - 1 ) * FPixelSpacing );
    charh := ( pix_y * psy ) + ( ( pix_y - 1 ) * FPixelSpacing );
    FNoOfChars := ( Width - ( 2 * FBorderSpace ) + FCharSpacing ) div ( charw + FCharSpacing );
    FTextLines := ( Height - ( 2 * FBorderSpace ) + FLineSpacing ) div ( charh + FLineSpacing );
    if FNoOfChars < 1 then FNoOfChars := 1;
    if FTextLines < 1 then FTextLines := 1;
    Width := ( FBorderSpace * 2 ) + { Distance to sides (there are two) }
      ( FCharSpacing * ( FNoOfChars - 1 ) ) + { Spaces between charactes          }
      ( charw * FNoOfChars ) + { The characters itself             }
      2; { For the border                    }
    Height := ( FBorderSpace * 2 ) + { Distance to top and bottom        }
      ( FLineSpacing * ( FTextLines - 1 ) ) + { Spacing between lines             }
      ( charh * FTextLines ) + { The lines                         }
      2; { For the border                    }
    FWidth := Width;
    FHeight := Height;
    if Assigned(BitMap) then BitMap.free; 
    BitMap := NewDIBBitmap( Width + 2, Height + 2, pf32bit );
   end;
end;

function THILedText.GetCaption;
var   Buf: array[ 0..255 ] of Char;
begin
   GetTextBuf( Buf, 256 );
   StrCopy( charbuf, Buf );
   Result := Buf;
end;

procedure THILedText.SetCaption;
var   Buffer: array[ 0..255 ] of Char;
begin
   if GetCaption <> Value then begin
      SetTextBuf( StrPCopy( Buffer, Value ) );
      StrCopy( charbuf, Buffer );
   end;
end;

function THILedText.GetTextBuf;
begin
  Result := Control.Perform( WM_GETTEXT, BufSize, Longint( Buffer ) );
end;

procedure THILedText.SetTextBuf;
const   CM_TEXTCHANGED = CM_BASE + 18;
begin
  Control.Perform( WM_SETTEXT, 0, Longint( Buffer ) );
  Control.Perform( CM_TEXTCHANGED, 0, 0 );
end;

procedure THILedText.DrawMatrix;
var   i, j: integer;
      tx, ty: integer;
      CColor: TColor;
begin
    tx := xpos;
    ty := ypos;
    CColor := FPixOffColor;
    for i := 1 to pix_y do begin
      for j := 1 to pix_x do begin
        case FDotMatrix of
          mat5x7: begin
             if ( i <= 7 ) then begin
                if Char5x7[ charindex ][ i ][ j ] = 1 then
                   CColor := FPixOnColor
                else
                   CColor := FPixOffColor;
             end else
                CColor := FPixOffColor;end;
          mat7x9: begin
             if ( i <= 9 ) then begin
                if Char7x9[ charindex ][ i ][ j ] = 1 then
                   CColor := FPixOnColor
                else
                   CColor := FPixOffColor;
             end else
                CColor := FPixOffColor;end;
          mat9x12: begin
             if ( i <= 12 ) then begin
                if Char9x12[ charindex ][ i ][ j ] = 1 then
                   CColor := FPixOnColor
                else
                   CColor := FPixOffColor;
             end else
                CColor := FPixOffColor;end;
          dos5x7: begin
             if ( i <= 7 ) then begin
                if CharDOS5x7[ charindex ][ i ][ j ] = 1 then
                   CColor := FPixOnColor
                else
                   CColor := FPixOffColor;
             end else
                CColor := FPixOffColor;end;
        end;
    // Paint pixels in selected shape
        case FPixelShape of
          spSquare: begin // Standard square pixels
              if FPixelSize = pix1x1 then
                Bitmap.Canvas.Pixels[ tx, ty ] := CColor
              else begin
                BitMap.Canvas.Pen.Color := CColor;
                BitMap.Canvas.Brush.Color := CColor;
                BitMap.Canvas.rectangle( tx, ty, tx + psx, ty + psy );
              end;
            end;
          spShaped: begin // Pixels with shaped corners
              if FPixelSize = pix1x1 then
                Bitmap.Canvas.Pixels[ tx, ty ] := CColor
              else begin
                if CColor = FPixOnColor then begin
                  BitMap.Canvas.Pen.Color := FPixHalfColor;
                  BitMap.Canvas.Brush.Color := FpixHalfColor;
                  BitMap.Canvas.rectangle( tx, ty, tx + psx, ty + psy );
                  BitMap.Canvas.Pen.Color := CColor;
                  BitMap.Canvas.Brush.Color := CColor;
                  BitMap.Canvas.ellipse( tx, ty, tx + psx, ty + psy );
                end else begin
                  BitMap.Canvas.Pen.Color := CColor;
                  BitMap.Canvas.Brush.Color := CColor;
                  BitMap.Canvas.rectangle( tx, ty, tx + psx, ty + psy );
                end;
              end;
            end;
          spRound: begin // Round pixels
              if FPixelSize = pix1x1 then
                Bitmap.Canvas.Pixels[ tx, ty ] := CColor
              else begin
                BitMap.Canvas.Pen.Color := CColor;
                BitMap.Canvas.Brush.Color := CColor;
                BitMap.Canvas.ellipse( tx, ty, tx + psx, ty + psy );
              end;
            end;
        end;
        tx := tx + psx + FPixelSpacing;
      end;
      tx := xpos;
      ty := ty + psy + FPixelSpacing;
    end;
end;

procedure THILedText.DrawCharacters( BitMap: PBitMap );
var
  row, col: integer;
  xpos, ypos: integer;
  charindex: integer;
  cc: integer;
  textend: Boolean;
begin
    xpos := FBorderSpace + 1;
    ypos := FBorderSpace + 1;
    cc := 0;
    textend := False;
    for row := 1 to FTextLines do begin { Line counter             }
      for col := 1 to FNoOfChars do begin { Character counter        }
        if textend = False then { Check for string end     }
          if charbuf[ cc ] = #0 then
            textend := True;
        if textend then
          charindex := 0
        else
          charindex := Ord( charbuf[ cc ] );
        if ( charindex < first_c ) then { Limit charactes inside interval }
          charindex := first_c;
        if ( charindex > last_c ) then
          charindex := last_c;
        DrawMatrix( BitMap, xpos, ypos, charindex );
        xpos := xpos + charw + FCharSpacing;
        Inc( cc );
      end;
      ypos := ypos + charh + FLineSpacing;
      xpos := FBorderSpace + 1;
    end;
end;

end.